/******************** (C) COPYRIGHT 2024 GeekRebot *****************************
* File Name          : show.c
* Current Version    : V1.0  & ST 3.5.0
* Author             : zhanli 719901725@qq.com
* Date of Issued     : 2024.01.13 zhanli: Create
* Comments           : GeekRebot STM32
********************************************************************************/
#include "filter.h"


float angle,angle_dot; 	
float Q_angle = 0.001;   // 过程噪声的协方差
float Q_gyro  = 0.003;   // 0.003 过程噪声的协方差 过程噪声的协方差为一个一行两列矩阵
float R_angle = 0.5;     // 测量噪声的协方差 既测量偏差
float dt      = 0.005;   // 积分时间，这个IMU更新的周期有关                 
char  C_0     = 1;

float Q_bias, Angle_err;
float PCt_0, PCt_1, E;
float K_0, K_1, t_0, t_1;
float Pdot[4]  = {0,0,0,0};
float PP[2][2] = { { 1, 0 },{ 0, 1 } };

/**----------------------------------------------------------------------
* Function    : Kalman_Filter
* Description : 一阶互补滤波
* Input       : 加速度、角速度
* Author      : zhanli&719901725@qq.com
* Date        : 2024/01/13 zhanli
*---------------------------------------------------------------------**/ 
void KalmanFilter(float acc,float gyro)		
{
	angle  += (gyro - Q_bias) * dt; //先验估计
	Pdot[0] = Q_angle - PP[0][1] - PP[1][0]; // Pk-先验估计误差协方差的微分

	Pdot[1] = -PP[1][1];
	Pdot[2] = -PP[1][1];
	Pdot[3] = Q_gyro;

	PP[0][0] += Pdot[0] * dt;   // Pk-先验估计误差协方差微分的积分
	PP[0][1] += Pdot[1] * dt;   // =先验估计误差协方差
	PP[1][0] += Pdot[2] * dt;
	PP[1][1] += Pdot[3] * dt;
		
	Angle_err = acc - angle;	//zk-先验估计
	
	PCt_0 = C_0 * PP[0][0];
	PCt_1 = C_0 * PP[1][0];
	
	E = R_angle + C_0 * PCt_0;
	
	K_0 = PCt_0 / E;
	K_1 = PCt_1 / E;
	
	t_0 = PCt_0;
	t_1 = C_0 * PP[0][1];

	PP[0][0] -= K_0 * t_0;		     // 后验估计误差协方差
	PP[0][1] -= K_0 * t_1;
	PP[1][0] -= K_1 * t_0;
	PP[1][1] -= K_1 * t_1;
		
	angle	 += K_0 * Angle_err;	 // 后验估计
	Q_bias	 += K_1 * Angle_err;	 // 后验估计
	angle_dot = gyro - Q_bias;	     // 输出值(后验估计)的微分 = 角速度
}

/**----------------------------------------------------------------------
* Function    : First_order_filter
* Description : 一阶互补滤波
* Input       : 加速度、角速度
* Author      : zhanli&719901725@qq.com
* Date        : 2024/01/13 zhanli
*---------------------------------------------------------------------**/ 
void First_order_filter(float angle, float gyro)
{
    float alpha = 0.02f;                        // 低通滤波参数
    float DeltaAngle = gyro * 0.005;            // 角度增量
    angle = alpha * angle + (1 - alpha) * (angle + DeltaAngle);
}
